@echo off
rem vMix to Transport Stream for Portsdown (LimeDVB Mode)
rem By Dave G8GKQ based on an original by Chris MW0LLK with 
rem contributions from Rob M0DTS and Ray M0DHP

rem Set the IP address of your Portsdown TX here

set ip=192.168.2.214

rem Set your callsign here

set callsign=G8GKQ

rem Print banner
echo vMix H.264 Video Encoder for the BATC Portsdown Transmitter
echo ===========================================================
echo:
echo Portsdown IP: %ip%  Your callsign set to %callsign%
echo:
echo Start this script once vMix is running with the external output on, 
echo then leave it running.  Then select Transmit on the Portsdown
echo:
echo This script is limited to exact SRs of 250kS, 333kS, 500kS and 1000kS
echo Set the Modulation type, SR, FEC and coding headroom
echo Press Enter for default (previous) values
echo:

:requestMod
if X%MOD%==X set MOD=2
set defMOD=%MOD%
set /p MOD=Enter Modulation (1 for DVB-S, 2 for DVB-S2) (%MOD%):

:requestSR
if X%SR%==X set SR=333
set defSR=%SR%
set /p SR=Enter SR (250, 333, 500 or 1000) in kS (%SR%):

set _tempvar=0
if %SR%==250  set _tempvar=1
if %SR%==333  set _tempvar=1
if %SR%==500  set _tempvar=1
if %SR%==1000 set _tempvar=1
if NOT %_tempvar% EQU 1 goto :requestSR

if X%SR%==X set SR=%defSR%

:requestFEC
if %MOD%==1 echo Available FECs are 1/2, 2/3, 3/4, 5/6, or 7/8
if %MOD%==2 echo Available FECs are 1/2, 2/3, 3/4, 5/6, or 9/10

if X%FEC%==X set FEC=1/2
set defFEC=%FEC%
set /p FEC=Enter FEC (%FEC%):

set _tempvar=0
if %FEC%==1/2 set _tempvar=1
if %FEC%==2/3 set _tempvar=1
if %FEC%==3/4 set _tempvar=1
if %FEC%==5/6 set _tempvar=1
if %FEC%==7/8 set _tempvar=1
if %FEC%==9/10 set _tempvar=1
if NOT %_tempvar% EQU 1 goto :requestFEC

if X%FEC%==X set FEC=%defFEC%

:requestHEADROOM
if X%HEADROOM%==X set HEADROOM=35
set defHEADROOM=%HEADROOM%
set /p HEADROOM=Enter Coding Headroom (0 to 100 percent, 35 suggested) (%HEADROOM%):

rem Save values for next time
rem If these lines cause errors your system may not support SETX
rem If so just comment them out (use rem)

if not %MOD%==%defMOD% setx MOD "%MOD%"
if not %SR%==%defSR% setx SR "%SR%"
if not %FEC%==%defFEC% setx FEC "%FEC%"
if not %HEADROOM%==%defHEADROOM% setx HEADROOM "%HEADROOM%"

rem Look up Video bit rate value for current SR/FEC/MOD

set IMAGESIZE=768x576

if %MOD%==1 (
  rem DVB-S
  if %SR%==250 (
    set IMAGESIZE=384x288
    if %FEC%==1/2 set VIDRATE=230000
    if %FEC%==2/3 set VIDRATE=307200
    if %FEC%==3/4 set VIDRATE=345600
    if %FEC%==5/6 set VIDRATE=384000
    if %FEC%==7/8 set VIDRATE=403200
  )
  if %SR%==333 (
    set IMAGESIZE=384x288
    if %FEC%==1/2 set VIDRATE=306900
    if %FEC%==2/3 set VIDRATE=409200
    if %FEC%==3/4 set VIDRATE=460300
    if %FEC%==5/6 set VIDRATE=511500
    if %FEC%==7/8 set VIDRATE=537000
  )
  if %SR%==500 (
    if %FEC%==1/2 set VIDRATE=460800
    if %FEC%==2/3 set VIDRATE=614400
    if %FEC%==3/4 set VIDRATE=691200
    if %FEC%==5/6 set VIDRATE=768000
    if %FEC%==7/8 set VIDRATE=806400
  )
  if %SR%==1000 (
    if %FEC%==1/2 set VIDRATE=921600
    if %FEC%==2/3 set VIDRATE=1228800
    if %FEC%==3/4 set VIDRATE=1382400
    if %FEC%==5/6 set VIDRATE=1535900
    if %FEC%==7/8 set VIDRATE=1612700
  )
)

if %MOD%==2 (
  rem DVB-S2
  if %SR%==250 (
    set IMAGESIZE=384x288
    if %FEC%==1/2 set VIDRATE=247500
    if %FEC%==2/3 set VIDRATE=330600
    if %FEC%==3/4 set VIDRATE=372100
    if %FEC%==5/6 set VIDRATE=413700
    if %FEC%==9/10 set VIDRATE=446900
  )
  if %SR%==333 (
    set IMAGESIZE=384x288
    if %FEC%==1/2 set VIDRATE=329600
    if %FEC%==2/3 set VIDRATE=440300
    if %FEC%==3/4 set VIDRATE=495700
    if %FEC%==5/6 set VIDRATE=551000
    if %FEC%==9/10 set VIDRATE=595300
  )
  if %SR%==500 (
    if %FEC%==1/2 set VIDRATE=494900
    if %FEC%==2/3 set VIDRATE=661100
    if %FEC%==3/4 set VIDRATE=744200
    if %FEC%==5/6 set VIDRATE=827300
    if %FEC%==9/10 set VIDRATE=893800
  )
  if %SR%==1000 (
    if %FEC%==1/2 set VIDRATE=989800
    if %FEC%==2/3 set VIDRATE=1322300
    if %FEC%==3/4 set VIDRATE=1488500
    if %FEC%==5/6 set VIDRATE=1654700
    if %FEC%==9/10 set VIDRATE=1787600
  )
)

set BASEVIDRATE=%VIDRATE%

rem take account of audio and then factor in headroom
set /a VIDRATE=(%BASEVIDRATE%-30000)*100/(100+%HEADROOM%)

set /a BUFSIZE=%VIDRATE% * 2

echo:
if %MOD%==1 echo For DVB-S at %SR% kS and FEC %FEC% the max datarate is %BASEVIDRATE% 
if %MOD%==2 echo For DVB-S2 at %SR% kS and FEC %FEC% the max datarate is %BASEVIDRATE% 
echo:
echo With a headroom of %HEADROOM% percent the video bitrate set is %VIDRATE%

echo:
echo|set /p="TS stream will start when you "

pause

start "Video Transport Stream to Portsdown" /high ^
C:\ffmpeg\bin\ffmpeg ^
-rtbufsize 1500M ^
-f dshow -i video="vMix Video" -f dshow -i audio="vMix Audio" ^
-c:v libx264 -x264-params "nal-hrd=cbr" -preset ultrafast -pix_fmt yuv420p -r 25 -s %IMAGESIZE% -aspect 4:3 -qmin 2 -qmax 35 ^
-b:v %VIDRATE% -minrate %VIDRATE% -maxrate %VIDRATE% -bufsize %BUFSIZE% ^
-c:a aac ^
-b:a 24000 -ar 48000 -ac 1 ^
-f mpegts -mpegts_original_network_id 1 -mpegts_transport_stream_id 1 -mpegts_service_id 1 -mpegts_pmt_start_pid 4096 -streamid 0:256 -streamid 1:257 ^
-metadata service_provider="Portsdown IPTS" -metadata service_name=%callsign% ^
-flush_packets 0 -y udp://%ip%:10000?pkt_size=1316
